--[[============================================================================
main.lua
============================================================================]]--

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

local steps = 1
local selected_note_column

function next_line_from(sequence, line, direction) -- used for iteration. direction is 1 for forward and -1 for backwards
 local pattern = renoise.song().sequencer.pattern_sequence[sequence]
 if direction == 1 then
  if line == renoise.song().patterns[pattern].number_of_lines then
   if sequence + 1 > table.getn(renoise.song().sequencer.pattern_sequence) then
    return "END"
   else
    sequence, line = sequence + direction, 1
   end
  else
   line = line + 1
  end
 else
  if line == 1 then
   if sequence == 1 then
    return "END"
   else
    sequence = sequence - 1
    line = renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].number_of_lines
   end
  else
   line = line - 1
  end
 end
 return sequence, line, direction
end

function next_note_from(sequence, line, direction) -- finding the next/previous note from current line position
 local n_sequence, n_line, direction = next_line_from(sequence, line, direction)
 if n_sequence == "END" then return "END" else
  if renoise.song().patterns[renoise.song().sequencer.pattern_sequence[n_sequence]].tracks[renoise.song().selected_track_index]:line(n_line).note_columns[selected_note_column].note_value < 120 then
   return { n_sequence, n_line, steps }
  else
   steps = steps + 1
   return next_note_from(next_line_from(sequence, line, direction))
  end
 end
end

function round(x) -- not provided by lua math. :)
  if x%2 ~= 0.5 then
    return math.floor(x+0.5)
  end
  return x-0.5
end

function insert_slide_effect(lines, slide_per_line) -- insertion of slide effect
 local sequence, line = renoise.song().selected_sequence_index, renoise.song().selected_line_index
 local slide_direction = 0
 if slide_per_line > 0 then slide_direction = 1 else slide_direction = 2 end
 slide_per_line = math.abs(slide_per_line)
 local actual_slide_accumulated = 0
 local float_slide_accumulated = 0
 local this_slide = round(slide_per_line)
 for i = 1, lines do
  if this_slide > 0 then
   local effect_val
   if slide_direction == 1 then effect_val = "0U" else effect_val = "0D" end
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].number_string = effect_val
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].amount_value = this_slide
  else
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].number_string = "00"
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].amount_value = 0
  end
  actual_slide_accumulated = actual_slide_accumulated + this_slide
  float_slide_accumulated = slide_per_line * i
  local rest = float_slide_accumulated - actual_slide_accumulated
  this_slide = round(slide_per_line+rest)
  sequence, line = next_line_from(sequence, line, 1)
  if i == lines then
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].number_string = "0G"
   renoise.song().patterns[renoise.song().sequencer.pattern_sequence[sequence]].tracks[renoise.song().selected_track_index]:line(line).effect_columns[1].amount_value = 0
  end
 end
end

function calculate_glide(next_note, previous_note) -- initial calculations for slide effect pattern data
 local semi_notes = renoise.song().patterns[renoise.song().sequencer.pattern_sequence[next_note[1]]].tracks[renoise.song().selected_track_index]:line(next_note[2]):note_column(selected_note_column).note_value - renoise.song().patterns[renoise.song().sequencer.pattern_sequence[previous_note[1]]].tracks[renoise.song().selected_track_index]:line(previous_note[2]):note_column(selected_note_column).note_value
 if semi_notes ~= 0 then
 local slide_per_line = semi_notes * 16 / next_note[3]
  insert_slide_effect(next_note[3], slide_per_line)
 end
end

function main()
 if renoise.song().selected_note_column_index == 0 then
  selected_note_column = 1
 else
  selected_note_column = renoise.song().selected_note_column_index
 end 
 if renoise.song().selected_track_index > renoise.song().sequencer_track_count then
  return
 else
  steps = 1
  local next_note = next_note_from(renoise.song().selected_sequence_index, renoise.song().selected_line_index, 1)
  steps = 1 -- resetting a counter
  local previous_note = next_note_from(renoise.song().selected_sequence_index, renoise.song().selected_line_index+1, -1)
  if next_note ~= "END" and previous_note ~= "END" then
   calculate_glide(next_note, previous_note)
  end
 end
end

--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Slide to next note",
  invoke = main
}


--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

renoise.tool():add_keybinding {
  name = "Pattern Editor:Track Operations:Slide to next note",
  invoke = main
}
